part of graphlib.alg;

/**
 * Finds all [connected components](http://en.wikipedia.org/wiki/Connected_component_(graph_theory))
 * in a graph and returns an array of these components. Each component is
 * itself an array that contains the ids of nodes in the component.
 *
 * This function only works with undirected Graphs.
 *
 * img(src="https://cpettitt.github.io/project/graphlib/latest/doc/static/components-source.png")
 *
 *     alg.components(graph);
 *     // => [ [ 'A', 'B', 'C', 'D' ],
 *     //      [ 'E', 'F', 'G' ],
 *     //      [ 'H', 'I' ] ]
 */
List components(BaseGraph g) {
  var results = [];
  var visited = new Set();

  dfs(v, List component) {
    if (!visited.contains(v)) {
      visited.add(v);
      component.add(v);
      g.neighbors(v).forEach((w) {
        dfs(w, component);
      });
    }
  }

  g.nodes().forEach((v) {
    var component = [];
    dfs(v, component);
    if (component.length > 0) {
      results.add(component);
    }
  });

  return results;
}
