part of graphlib.alg;

/**
 * Given a [Digraph] [g] this function returns all nodes that are part of a
 * cycle. Since there may be more than one cycle in a graph this function
 * returns an array of these cycles, where each cycle is itself represented
 * by an array of ids for each node involved in that cycle.
 *
 * [isAcyclic] is more efficient if you only need to determine whether
 * a graph has a cycle or not.
 *
 *     var digraph = new Digraph();
 *     digraph.addNode(1);
 *     digraph.addNode(2);
 *     digraph.addNode(3);
 *     digraph.addEdge(null, 1, 2);
 *     digraph.addEdge(null, 2, 3);
 *
 *     alg.findCycles(digraph);
 *     // => []
 *
 *     digraph.addEdge(null, 3, 1);
 *     alg.findCycles(digraph);
 *     // => [ [ 3, 2, 1 ] ]
 *
 *     digraph.addNode(4);
 *     digraph.addNode(5);
 *     digraph.addEdge(null, 4, 5);
 *     digraph.addEdge(null, 5, 4);
 *     alg.findCycles(digraph);
 *     // => [ [ 3, 2, 1 ], [ 5, 4 ] ]
 */
List findCycles(Digraph g) {
  return tarjan(g).where((cmpt) { return cmpt.length > 1; }).toList();
}
