part of graphlib.alg;

/**
 * This function performs a [preorder traversal](https://en.wikipedia.org/wiki/Tree_traversal#Depth-first)
 * of the graph [g] starting at the node [root]. For each node visited, `u`,
 * the function [f] is called. If the graph is a directed graph or if it does
 * not represent a tree structure, the function will throw an [Exception].
 * In the latter case, some nodes may be visited before it is determined that
 * the graph is not a tree.
 *
 * img(src="https://cpettitt.github.io/project/graphlib/latest/doc/static/preorder-source.png")
 *
 *     alg.preorder(graph, "A");
 *     // => One of:
 *     // [ "A", "B", "C", "D", "E" ]
 *     // [ "A", "B", "C", "E", "D" ]
 *     // [ "A", "C", "D", "E", "B" ]
 *     // [ "A", "C", "E", "D", "B" ]
 */
preorder(BaseGraph g, root, f(u)) {
  var visited = new Set();
  if (g.isDirected()) {
    throw new Exception("This function only works for undirected graphs");
  }
  dfs(u, [prev=null]) {
    if (visited.contains(u)) {
      throw new Exception("The input graph is not a tree: $g");
    }
    visited.add(u);
    f(u);
    g.neighbors(u).forEach((v) {
      if (v != prev) dfs(v, u);
    });
  }
  dfs(root);
}
