part of graphlib.test;

abstractBaseGraphTest(BaseGraph makeGraph()) {
  BaseGraph g;

  //group('BaseGraph', () {
    setUp(() {
      g = makeGraph();
    });

    group('new graph', () {
      test('has no nodes', () {
        expect(g.order(), equals(0));
        expect(g.nodes().length, equals(0));
      });

      test('has no edges', () {
        expect(g.size(), equals(0));
        expect(g.edges().length, equals(0));
      });
    });

    group('order', () {
      test('returns the number of nodes in the graph', () {
        var g = makeGraph();
        g.addNode('a');
        g.addNode('b');
        expect(g.order(), equals(2));
      });
    });

    group('size', () {
      test('returns the number of edges in the graph', () {
        var g = makeGraph();
        g.addNode('a');
        g.addNode('b');

        g.addEdge(null, 'a', 'b');
        expect(g.size(), equals(1));

        g.addEdge(null, 'a', 'b');
        expect(g.size(), equals(2));
      });
    });

    group('graph', () {
      test('returns undefined if a value has not been set', () {
        expect(g.graph(), isNull);
      });

      test('sets the value for the graph if called with a single argument', () {
        g.graph('test');
        expect(g.graph(), equals('test'));
      });
    });

    group('node', () {
      test('throws if the node isn\'t in the graph', () {
        expect(() { g.node(1); }, throws);
      });

      test('has an undefined value if no value was assigned to the node', () {
        g.addNode(1);
        expect(g.node(1), isNull);
      });

      test('returns the node\'s value if one was set', () {
        var value = { 'abc': 123 };
        g.addNode(1, value);
        expect(g.node(1), same(value));
      });

      test('sets the value for the node if called with two arguments', () {
        g.addNode(1);
        g.node(1, 'abc');
        expect(g.node(1), equals('abc'));
      });
    });

    group('edge', () {
      test('throws if the edge isn\'t in the graph', () {
        expect(() { g.edge(3); }, throws);
      });

      test('has the nodes that are incident on the edge', () {
        g.addNode(1);
        g.addNode(2);
        g.addEdge(3, 1, 2);
        expect(g.incidentNodes(3)..sort(), equals([1, 2]));
      });

      test('has an undefined value if no value was assigned to the edge', () {
        g.addNode(1);
        g.addNode(2);
        g.addEdge(3, 1, 2);
        expect(g.edge(3), isNull);
      });

      test('returns the edge\'s value if one was set', () {
        var value = { 'abc': 123 };
        g.addNode(1);
        g.addNode(2);
        g.addEdge(3, 1, 2, value);
        expect(g.edge(3), same(value));
      });

      test('sets the value for the edge if called with 2 arguments', () {
        g.addNode(1);
        g.addNode(2);
        g.addEdge(3, 1, 2);
        g.edge(3, 'abc');
        expect(g.edge(3), equals('abc'));
      });
    });

    group('edges', () {
      test('returns all edges with no arguments', () {
        g.addNode(1);
        g.addNode(2);
        g.addEdge('A', 1, 2);
        g.addEdge('B', 1, 2);
        g.addEdge('C', 2, 1);

        expect(g.edges()..sort(), equals(['A', 'B', 'C']));
      });
    });

    group('incidentEdges', () {
      test('returns all edges incident on a node', () {
        g.addNode(1);
        g.addNode(2);
        g.addNode(3);
        g.addEdge('A', 1, 2);
        g.addEdge('B', 1, 2);
        g.addEdge('C', 1, 1);
        g.addEdge('D', 2, 1);
        g.addEdge('E', 2, 3);

        expect(g.incidentEdges(1)..sort(), equals(['A', 'B', 'C', 'D']));
        expect(g.incidentEdges(2)..sort(), equals(['A', 'B', 'D', 'E']));
        expect(g.incidentEdges(3)..sort(), equals(['E']));
      });

      test('optional returns all edges between two nodes', () {
        g.addNode(1);
        g.addNode(2);
        g.addNode(3);
        g.addEdge('A', 1, 2);
        g.addEdge('B', 1, 2);
        g.addEdge('C', 1, 1);
        g.addEdge('D', 2, 1);
        g.addEdge('E', 2, 3);

        expect(g.incidentEdges(1, 1)..sort(), equals(['C']));
        expect(g.incidentEdges(1, 2)..sort(), equals(['A', 'B', 'D']));
        expect(g.incidentEdges(1, 3)..sort(), equals([]));
        expect(g.incidentEdges(2, 1)..sort(), equals(['A', 'B', 'D']));
        expect(g.incidentEdges(2, 2)..sort(), equals([]));
        expect(g.incidentEdges(2, 3)..sort(), equals(['E']));
        expect(g.incidentEdges(3, 1)..sort(), equals([]));
        expect(g.incidentEdges(3, 2)..sort(), equals(['E']));
        expect(g.incidentEdges(3, 3)..sort(), equals([]));
      });
    });

    group('toString', () {
      test('returns a string representation of the graph', () {
        // Just make sure we don't throw an Error
        g.addNode('a');
        g.addNode('b');
        g.addEdge(null, 'a', 'b');
        g.toString();
      });
    });

    group('addNode', () {
      test('adds a new node to the graph', () {
        g.addNode(1);
        expect(g.nodes(), equals([1]));
        expect(g.neighbors(1).length, equals(0));
      });

      test('assigns an arbitrary id to the node if the given id is undefined', () {
        var id = g.addNode();
        expect(id, isNotNull);
        expect(g.nodes(), equals([id]));
        expect(g.neighbors(id).length, equals(0));
      });

      test('fails if the node is already in the graph', () {
        g.addNode('foo');
        expect(() { g.addNode('foo'); }, throws, reason: "Graph already has node 'foo'");
      });
    });

    group('delNode', () {
      test('removes the node from the graph', () {
        g.addNode(1);
        g.addNode(2);
        g.delNode(1);
        expect(g.nodes(), equals([2]));
      });

      test('removes out edges', () {
        g.addNode(1);
        g.addNode(2);
        g.addEdge('1 -> 2', 1, 2);
        g.delNode(1);
        expect(g.neighbors(2).length, equals(0));
      });

      test('removes in edges', () {
        g.addNode(1);
        g.addNode(2);
        g.addEdge('2 -> 1', 2, 1);
        g.delNode(1);
        expect(g.neighbors(2).length, equals(0));
      });
    });

    group('addEdge', () {
      test('adds a new edge to the graph', () {
        g.addNode(1);
        g.addNode(2);
        g.addEdge('a', 1, 2);

        expect(g.edges(), equals(['a']));

        expect(g.neighbors(1), equals([2]));
        expect(g.neighbors(2), equals([1]));
      });

      test('assigns an arbitrary edge id if the given id is null', () {
        g.addNode(1);
        g.addNode(2);
        g.addNode(3);

        var e1 = g.addEdge(null, 1, 2);
        expect(e1, isNotNull);
        expect(g.hasEdge(e1), isTrue);

        var e2 = g.addEdge(null, 2, 3);
        expect(e2, isNotNull);
        expect(g.hasEdge(e2), isTrue);

        expect(g.neighbors(1), equals([2]));
        expect(g.neighbors(2), unorderedEquals([1, 3]));
        expect(g.neighbors(3), equals([2]));
      });

      /*test('assigns an arbitrary edge id if the given id is undefined', () {
        g.addNode(1);
        g.addNode(2);
        g.addNode(3);

        var e1 = g.addEdge(undefined, 1, 2);
        expect(e1, isNotNull);
        expect(g.hasEdge(e1), isTrue);

        var e2 = g.addEdge(undefined, 2, 3);
        expect(e2, isNotNull);
        expect(g.hasEdge(e2), isTrue);

        expect(g.neighbors(1), same([2]));
        expect(g.neighbors(2), same([1, 3]));
        expect(g.neighbors(3), same([2]));
      });*/

      test('avoids id collisions when given id is null', () {
        g.addNode(1);
        g.addNode(2);

        // We use this assert in case the internal id scheme changes...
        expect(g.addEdge(null, 1, 2), equals('_1'));

        // This would be the next id assigned by the graph
        g.addEdge('_2', 1, 2);

        expect(g.addEdge(null, 1, 2), equals('_3'));
      });

      test('fails if the edge is already in the graph', () {
        g.addNode(1);
        g.addNode(2);
        g.addEdge('foo', 1, 2);
        expect(() { g.addEdge('foo', 1, 2); }, throws, reason: "Graph already has edge 'foo'");
      });
    });

    group('delEdge', () {
      test('removes the specified edge from the graph', () {
        g.addNode(1);
        g.addNode(2);
        g.addEdge('a', 1, 2);
        g.delEdge('a');

        expect(g.edges().length, equals(0));

        expect(g.neighbors(1).length, equals(0));
      });
    });

    group('copy', () {
      test('copies the graph value', () {
        g.graph('foo');
        expect(g.copy().graph(), equals('foo'));
      });

      test('copies the nodes in the graph', () {
        g.addNode(1);
        g.addNode(2, {'foo': 'bar'});
        var copy = g.copy();
        expect(copy.nodes(), equals(g.nodes()));
        expect(copy.node(2), equals(g.node(2)));
      });

      test('copies the edges in the graph', () {
        g.addNode(1);
        g.addNode(2);
        g.addEdge('A', 1, 2, {'foo': 'bar'});
        var copy = g.copy();
        expect(copy.edges(), equals(g.edges()));
        expect(copy.edge('A'), equals(g.edge('A')));
      });

//      test('deepEquals the original graph', () {
//        g.addNode(1);
//        g.addNode(2);
//        // The `null` here is important because we want to increment the nextId
//        // counter.
//        g.addEdge(null, 1, 2, {'foo': 'bar'});
//        var copy = g.copy();
//        expect(copy, equals(g));
//      });
    });

    group('filterNodes', () {
      test('creates a graph containing only nodes where the filter is true', () {
        var g = makeGraph();
        [1,2,3].forEach((u) { g.addNode(u); });

        var g2 = g.filterNodes(nodesFromList([1, 2]));
        expect(g2.nodes()..sort(), equals([1, 2]));
      });

//      test('creates a graph of the same type', () {
//        var g = makeGraph();
//        [1,2,3].forEach((u) { g.addNode(u); });
//        expect(g.filterNodes(filter.all()), instanceOf(makeGraph));
//      });

      test('includes the values of the filtered nodes', () {
        var g = makeGraph();
        [1,2,3].forEach((u) { g.addNode(u, 'V$u'); });

        var g2 = g.filterNodes(nodesFromList([1, 2]));
        expect(g2.node(1), equals('V1'));
        expect(g2.node(2), equals('V2'));
      });

      test('includes edges that are are incident with nodes in the filtered set', () {
        var g = makeGraph();
        [1,2,3].forEach((u) { g.addNode(u); });
        g.addEdge('a', 1, 2, 'VA');
        g.addEdge('b', 2, 3);

        var g2 = g.filterNodes(nodesFromList([1, 2]));
        expect(g2.hasEdge('a'), isTrue);
        expect(g2.edge('a'), equals('VA'));
      });
    });
//  });
}
