part of graphlib.alg.test;

//var assert = require('chai').assert,
//    Digraph = require('../..').Digraph,
//    Graph = require('../..').Graph;
//
//module.exports = allShortestPaths;

// For testing shortest path algorithms.
allShortestPaths(var alg) {
  test('returns 0 for the node itself', () {
    var g = new Digraph();
    g.addNode(1);
    expect(alg(g), equals({ 1: { 1: { 'distance': 0 } }}));
  });

  test('does not include positive weight self edges', () {
    var g = new Digraph();
    g.addNode(1);
    g.addEdge(null, 1, 1);
    expect(alg(g), equals({ 1: { 1: { 'distance': 0 } }}));
  });

  test('returns double.INFINITY for unconnected nodes', () {
    var g = new Digraph();
    g.addNode(1);
    g.addNode(2);
    expect(alg(g), equals({
      1: {
        1: { 'distance': 0 },
        2: { 'distance': double.INFINITY }
      },
      2: {
        1: { 'distance': double.INFINITY },
        2: { 'distance': 0 }
      }
    }));
  });

  test('returns the distance and path from all nodes to other nodes', () {
    var g = new Digraph();
    g.addNode(1);
    g.addNode(2);
    g.addNode(3);
    g.addEdge(null, 1, 2);
    g.addEdge(null, 2, 3);
    expect(alg(g), equals({
      1: {
        1: { 'distance': 0 },
        2: { 'distance': 1, 'predecessor': 1 },
        3: { 'distance': 2, 'predecessor': 2 }
      },
      2: {
        1: { 'distance': double.INFINITY },
        2: { 'distance': 0 },
        3: { 'distance': 1, 'predecessor': 2 }
      },
      3: {
        1: { 'distance': double.INFINITY },
        2: { 'distance': double.INFINITY },
        3: { 'distance': 0 }
      }
    }));
  });

  test('uses an optionally supplied weight function', () {
    var g = new Digraph();
    g.addNode(1);
    g.addNode(2);
    g.addNode(3);
    g.addEdge(null, 1, 2, 2);
    g.addEdge(null, 2, 3, 3);

    w(e) { return g.edge(e); }

    expect(alg(g, w), equals({
      1: {
        1: { 'distance': 0 },
        2: { 'distance': 2, 'predecessor': 1 },
        3: { 'distance': 5, 'predecessor': 2 }
      },
      2: {
        1: { 'distance': double.INFINITY },
        2: { 'distance': 0 },
        3: { 'distance': 3, 'predecessor': 2 }
      },
      3: {
        1: { 'distance': double.INFINITY },
        2: { 'distance': double.INFINITY },
        3: { 'distance': 0 }
      }
    }));
  });

  test('finds the shortest edge with multiple edges', () {
    var g = new Digraph();
    g.addNode(1);
    g.addNode(2);
    g.addEdge(null, 1, 2, 3);
    g.addEdge(null, 1, 2, 2);
    g.addEdge(null, 1, 2, 5);

    w(e) { return g.edge(e); }

    expect(alg(g, w), equals({
      1: {
        1: { 'distance': 0 },
        2: { 'distance': 2, 'predecessor': 1 }
      },
      2: {
        1: { 'distance': double.INFINITY },
        2: { 'distance': 0 }
      }
    }));
  });

  test('uses an optionally supplied incident function', () {
    var g = new Digraph();
    g.addNode(1);
    g.addNode(2);
    g.addNode(3);
    g.addEdge(null, 1, 2);
    g.addEdge(null, 2, 3);

    expect(alg(g, null/*undefined*/, (e) { return g.inEdges(e); }), equals({
      1: {
        1: { 'distance': 0 },
        2: { 'distance': double.INFINITY },
        3: { 'distance': double.INFINITY }
      },
      2: {
        1: { 'distance': 1, 'predecessor': 2 },
        2: { 'distance': 0 },
        3: { 'distance': double.INFINITY }
      },
      3: {
        1: { 'distance': 2, 'predecessor': 2 },
        2: { 'distance': 1, 'predecessor': 3 },
        3: { 'distance': 0 }
      }
    }));
  });

  test('works with undirected graphs', () {
    var g = new Graph();
    g.addNode(1);
    g.addNode(2);
    g.addNode(3);
    g.addNode(4);
    g.addEdge(null, 1, 2, 1);
    g.addEdge(null, 2, 3, 2);
    g.addEdge(null, 3, 1, 4);
    g.addEdge(null, 2, 4, 6);

    w(e) { return g.edge(e); }

    expect(alg(g, w), equals({
      1: {
        1: { 'distance': 0 },
        2: { 'distance': 1, 'predecessor': 1 },
        3: { 'distance': 3, 'predecessor': 2 },
        4: { 'distance': 7, 'predecessor': 2 },
      },
      2: {
        1: { 'distance': 1, 'predecessor': 2 },
        2: { 'distance': 0 },
        3: { 'distance': 2, 'predecessor': 2 },
        4: { 'distance': 6, 'predecessor': 2 },
      },
      3: {
        1: { 'distance': 3, 'predecessor': 2 },
        2: { 'distance': 2, 'predecessor': 3 },
        3: { 'distance': 0 },
        4: { 'distance': 8, 'predecessor': 2 },
      },
      4: {
        1: { 'distance': 7, 'predecessor': 2 },
        2: { 'distance': 6, 'predecessor': 4 },
        3: { 'distance': 8, 'predecessor': 2 },
        4: { 'distance': 0 },
      }
    }));
  });
}
