part of graphlib.alg.test;

//var assert = require('chai').assert,
//    Digraph = require('../..').Digraph,
//    Graph = require('../..').Graph,
//    dijkstra = require('../..').alg.dijkstra;

dijkstraTest() {
  group('alg.dijkstra', () {
    test('returns 0 for the node itself', () {
      var g = new Digraph();
      g.addNode(1);
      expect(dijkstra(g, 1), { 1: { 'distance': 0 } });
    });

    test('returns double.INFINITY for unconnected nodes', () {
      var g = new Digraph();
      g.addNode(1);
      g.addNode(2);
      expect(dijkstra(g, 1), equals({
        1: { 'distance': 0 },
        2: { 'distance': double.INFINITY }
      }));
    });

    test('returns the distance and path from the source node to other nodes', () {
      var g = new Digraph();
      g.addNode(1);
      g.addNode(2);
      g.addNode(3);
      g.addNode(4);
      g.addEdge(null, 1, 2);
      g.addEdge(null, 2, 3);
      g.addEdge(null, 2, 4);
      expect(dijkstra(g, 1), equals({
        1: { 'distance': 0 },
        2: { 'distance': 1, 'predecessor': 1 },
        3: { 'distance': 2, 'predecessor': 2 },
        4: { 'distance': 2, 'predecessor': 2 }
      }));
    });

    test('works for undirected graphs', () {
      var g = new Graph();
      g.addNode(1);
      g.addNode(2);
      g.addNode(3);
      g.addNode(4);
      g.addEdge(null, 1, 2);
      g.addEdge(null, 2, 3);
      g.addEdge(null, 2, 4);
      expect(dijkstra(g, 1), equals({
        1: { 'distance': 0 },
        2: { 'distance': 1, 'predecessor': 1 },
        3: { 'distance': 2, 'predecessor': 2 },
        4: { 'distance': 2, 'predecessor': 2 }
      }));
    });

    test('uses an optionally supplied weight function', () {
      var g = new Digraph();
      g.addNode(1);
      g.addNode(2);
      g.addNode(3);
      g.addNode(4);
      g.addEdge(null, 1, 2, 1);
      g.addEdge(null, 1, 3, 2);
      g.addEdge(null, 2, 4, 3);
      g.addEdge(null, 3, 4, 3);

      w(e) { return g.edge(e); }

      expect(dijkstra(g, 1, w), equals({
        1: { 'distance': 0 },
        2: { 'distance': 1, 'predecessor': 1 },
        3: { 'distance': 2, 'predecessor': 1 },
        4: { 'distance': 4, 'predecessor': 2 }
      }));
    });

    test('uses an optionally supplied incident function', () {
      var g = new Digraph();
      g.addNode(1);
      g.addNode(2);
      g.addNode(3);
      g.addNode(4);
      g.addEdge(null, 1, 3);
      g.addEdge(null, 2, 3);
      g.addEdge(null, 3, 4);

      expect(dijkstra(g, 4, null/*undefined*/, (e) { return g.inEdges(e); }), equals({
        1: { 'distance': 2, 'predecessor': 3 },
        2: { 'distance': 2, 'predecessor': 3 },
        3: { 'distance': 1, 'predecessor': 4 },
        4: { 'distance': 0 }
      }));
    });

    test('throws an Error if it encounters a negative edge weight', () {
      var g = new Digraph();
      g.addNode(1);
      g.addNode(2);
      g.addNode(3);
      g.addNode(4);
      g.addEdge(null, 1, 2, 1);
      g.addEdge(null, 1, 3, -2);
      g.addEdge(null, 2, 4, 3);
      g.addEdge(null, 3, 4, 3);

      w(e) { return g.edge(e); }

      expect(() { dijkstra(g, 1, w); }, throws);
    });
  });
}