part of graphlib.alg.test;

//var assert = require('../assert'),
//    Graph = require('../..').Graph,
//    prim = require('../..').alg.prim;

edgeWeight(g) {
  return (e) {
    return g.hasEdge(e) ? g.edge(e) : double.INFINITY;
  };
}

primTest() {
  group('alg.prim', () {
    test('returns an empty graph for an empty input', () {
      var source = new Graph();

      var g = prim(source, edgeWeight(source));
      expect(g.order(), equals(0));
      expect(g.size(), equals(0));
    });

    test('returns a single node graph for a graph with a single node', () {
      var source = new Graph();
      source.addNode(1);

      var g = prim(source, edgeWeight(source));
      expect(g.nodes(), equals([1]));
      expect(g.size(), equals(0));
    });

    test('returns a deterministic result given an optimal solution', () {
      var source = new Graph();
      source.addNode(1);
      source.addNode(2);
      source.addNode(3);
      source.addNode(4);
      source.addNode(5);
      source.addEdge(null, 1, 2, 1);
      source.addEdge(null, 2, 3, 2);
      source.addEdge(null, 2, 4, 3);
      source.addEdge(null, 3, 4, 20); // This edge should not be in the min spanning tree
      source.addEdge(null, 3, 5, 60); // This edge should not be in the min spanning tree
      source.addEdge(null, 4, 5, 1);

      var g = prim(source, edgeWeight(source));
      expect(g.neighbors(1)..sort(), equals([2]));
      expect(g.neighbors(2)..sort(), equals([1, 3, 4]));
      expect(g.neighbors(3)..sort(), equals([2]));
      expect(g.neighbors(4)..sort(), equals([2, 5]));
      expect(g.neighbors(5)..sort(), equals([4]));
    });

    test('throws an Error for unconnected graphs', () {
      var source = new Graph();
      source.addNode(1);
      source.addNode(2);

      expect(() { prim(source, edgeWeight(source)); }, throws);
    });
  });
}