part of graphlib.alg.test;

//var assert = require('chai').assert,
//    Digraph = require('../..').Digraph,
//    tarjan = require('../..').alg.tarjan;

tarjanTest() {
  group('alg.tarjan', () {
    test('returns an empty array for an empty graph', () {
      expect(tarjan(new Digraph()), equals([]));
    });

    test('returns single node components if the graph has no strongly connected components', () {
      var g = new Digraph();
      g.addNode(1);
      g.addNode(2);
      g.addNode(3);
      g.addEdge(null, 1, 2);
      g.addEdge(null, 2, 3);

      expect(sort(tarjan(g)), equals([[1], [2], [3]]));
    });

    test('returns a single component for a cycle of 1 edge', () {
      var g = new Digraph();
      g.addNode(1);
      g.addNode(2);
      g.addEdge(null, 1, 2);
      g.addEdge(null, 2, 1);

      expect(sort(tarjan(g)), equals([[1, 2]]));
    });

    test('returns a single component for a triangle', () {
      var g = new Digraph();
      g.addNode(1);
      g.addNode(2);
      g.addNode(3);
      g.addEdge(null, 1, 2);
      g.addEdge(null, 2, 3);
      g.addEdge(null, 3, 1);

      expect(sort(tarjan(g)), equals([[1, 2, 3]]));
    });

    test('can find multiple components', () {
      var g = new Digraph();
      g.addNode(1);
      g.addNode(2);
      g.addNode(3);
      g.addNode(4);
      g.addNode(5);
      g.addNode(6);
      g.addEdge(null, 1, 2);
      g.addEdge(null, 2, 1);
      g.addEdge(null, 3, 4);
      g.addEdge(null, 4, 5);
      g.addEdge(null, 5, 3);

      expect(sort(tarjan(g)), equals([[1, 2], [3, 4, 5], [6]]));
    });

//    test('throws an Error for undirected graphs', () {
//      expect(() { isAcyclic(new Graph()); }, throws);
//    });
  });
}

// A helper that sorts components and their contents
//sort(cmpts) {
//  cmpts.forEach((cmpt) {
//    cmpt.sort((lhs, rhs) { return lhs - rhs; });
//  });
//  cmpts.sort((lhs, rhs) { return lhs[0] - rhs[0]; });
//  return cmpts;
//}
